package ru.dbotthepony.kstarbound.defs.tile

import com.google.common.collect.ImmutableMap
import it.unimi.dsi.fastutil.objects.Object2DoubleMap
import it.unimi.dsi.fastutil.objects.Object2DoubleMaps
import org.apache.logging.log4j.LogManager
import ru.dbotthepony.kstarbound.json.builder.JsonFactory

@JsonFactory
data class TileDamageConfig(
	val damageFactors: ImmutableMap<String, Double> = ImmutableMap.of(),
	val damageRecovery: Double = 1.0,
	val maximumEffectTime: Double = 1.5,
	val totalHealth: Double = 1.0,
	val harvestLevel: Int = 1,
) {
	val damageFactorsMapped: ImmutableMap<TileDamageType, Double> = damageFactors.entries.stream().map {
		var find = TileDamageType.entries.firstOrNull { e -> e.match(it.key) }

		if (find == null) {
			LOGGER.error("Unknown tile damage type ${it.key}!")
			find = TileDamageType.PROTECTED
		}

		find to it.value
	}.collect(ImmutableMap.toImmutableMap({ it.first }, { it.second }))

	fun damageDone(damage: TileDamage): Double {
		return (damageFactorsMapped[damage.type] ?: 1.0) * damage.amount
	}

	companion object {
		val EMPTY = TileDamageConfig()

		private val LOGGER = LogManager.getLogger()
	}
}
